<?php

/*------------------------------------------------------------------------
# com_invoices - Invoices for Joomla
# ------------------------------------------------------------------------
# author				JoomlaThat!
# copyright 			Copyright (C) 2015 joomlathat.com. All Rights Reserved.
# @license				http://www.gnu.org/licenses/gpl-2.0.html GNU/GPL
# Websites: 			http://www.joomlathat.com
# Technical Support:	Forum - http://www.joomlathat.com/forum
-------------------------------------------------------------------------*/

/** ensure this file is being included by a parent file */
defined('_JEXEC') or die('Restricted access');

jimport('joomla.plugin.plugin');

if(!defined('DS')){
  define('DS',DIRECTORY_SEPARATOR);
}

class plgInvoicesExpensemanager_integration extends JPlugin {

  function __construct(& $subject, $config) {
    parent::__construct($subject, $config);
    $this->loadLanguage( '', JPATH_ADMINISTRATOR );

    $lang = JFactory::getLanguage();
    $lang->load('com_finances', JPATH_SITE);
  }

  /**
  * Trigger listener for invoices
  *
  * @param $row. The invoice data
  * @return bool.
  */
  function onAfterSaveInvoice($row)
  {
    if(file_exists(JPATH_SITE.DS.'components'.DS.'com_finances'.DS.'helpers'.DS.'helpers.php')) {
      require_once(JPATH_SITE . DS . 'components' . DS . 'com_finances' . DS . 'helpers' . DS . 'helpers.php');

      switch($this->params->get('save_what', 'total')){
        case "total":
        if($row->type == 1) $this->createOnAfterSaveComplete($row);
        break;
        case "items":
        if($row->type == 1) $this->createOnAfterSaveParts($row);
        break;
      }
    }

    return true;
  }

  /**
  * Trigger listener for payments
  *
  * @param $row. The payment data
  * @return bool.
  */
  function onAfterSavePayment($row)
  {
    if(file_exists(JPATH_SITE.DS.'components'.DS.'com_finances'.DS.'helpers'.DS.'helpers.php')) {
      require_once(JPATH_SITE . DS . 'components' . DS . 'com_finances' . DS . 'helpers' . DS . 'helpers.php');

      if ($this->params->get('save_what', 'total') == "payments" && $row->payment_status == 1) {
        $db = JFactory::getDBO();

        if ($this->params->get('name_type', 'number') == 'name') {
          $query = ' SELECT inv.to_name FROM #__invoices_payments AS pay LEFT JOIN #__invoices_invoices AS inv '.
          ' ON inv.id = pay.invoice_id ' .
          ' WHERE pay.id = ' . $row->id;
          $db->setQuery($query);
          $item_name = "Payment " . $db->loadResult();
        } else {
          $query = ' SELECT inv.invoice_num FROM #__invoices_payments AS pay LEFT JOIN #__invoices_invoices AS inv '.
          ' ON inv.id = pay.invoice_id ' .
          ' WHERE pay.id = ' . $row->id;
          $db->setQuery($query);
          $item_name = "Payment inv. " . $db->loadResult();
        }

        $existing_id = 0;
        if ($this->params->get('update', '1')) {
          $query = ' SELECT id FROM #__finances_items ' .
          ' WHERE external_type = "com_invoices,3" AND external_id = ' . $row->id;
          $db->setQuery($query);
          $existing_id = $db->loadResult();
        }

        $data = array();
        $data['id'] = $existing_id;
        $data['sign'] = "";
        $data['amount'] = $row->payment_amount*(-1); //Expenses signs on the database are changed
        $data['name'] = $item_name;
        $data['date_item'] = $row->payment_datetime;
        $data['tax'] = "";
        $data['category_id'] = $this->params->get('category_id', '1');
        $data['project_id'] = $this->params->get('account_id', '1');
        $data['external_id'] = $row->id;
        $data['external_type'] = "com_invoices,3";
        $data['vat_id'] = "";
        $data['external_ref'] = $row->payment_id;
        //$data['user_id'] = "";
        $data['description'] = "";
        $data['long_desc'] = $row->payment_description;

        FinancesHelper::create_expense_item($data);
      }
    }

    return true;
  }

  /**
  * Saves one expense according to the total amount of the invoice.
  *
  * @param $row. The invoice data
  */
  private function createOnAfterSaveComplete($row)
  {
    $existing_id = 0;
    if ($this->params->get('update', '1')){
      $db = JFactory::getDBO();
      $query = 	' SELECT id FROM #__finances_items '.
      ' WHERE external_type = "com_invoices,1" AND external_id = ' . $row->id ;
      $db->setQuery( $query );
      $existing_id = $db->loadResult();
    }

    if ($this->params->get('name_type', 'number') == 'name') {
      $item_name = "Inv. " . $row->to_name;
    } else {
      $item_name = "Inv. num " . $row->invoice_num;
    }

    $data = array();
    $data['id'] = $existing_id ;
    $data['sign'] = "";
    $data['amount'] = $row->computed_total*(-1); //Expenses signs on the database are changed
    $data['name'] = $item_name;
    $data['date_item'] = $row->invoice_date;

    if($row->taxes != ""){
      $tax = $this->getTax($row->taxes);
      $data['tax'] = $tax;
    }

    $data['category_id'] = $this->params->get('category_id', '1');
    $data['project_id'] = $this->params->get('account_id', '1');
    $data['external_id'] = $row->id;
    $data['external_type'] = "com_invoices,1";
    $data['vat_id'] = $row->to_vatid;
    $data['external_ref'] = $row->invoice_num;
    //$data['user_id'] = "";
    $data['description'] = "";
    $data['long_desc'] = $row->notes;

    FinancesHelper::create_expense_item($data);
  }

  /**
  * Saves one expense for each item in the invoice. The total invoice amount is not saved.
  *
  * @param $row. The invoice data
  */
  private function createOnAfterSaveParts($row)
  {
    $db = JFactory::getDBO();
    $query = 	' SELECT * FROM #__invoices_items '.
    ' WHERE invoice_id = ' . $row->id ;
    $db->setQuery( $query );

    $items = $db->loadObjectList();

    foreach ($items as $item){
      $amount = ($item->value*$item->amount) - $item->discount;
      $amount = $amount * (-1); //Expenses signs on the database are changed

      if ($this->params->get('name_type', 'number') == 'name') {
        $query = ' SELECT to_name FROM #__invoices_invoices '.
        ' WHERE id = ' . $item->invoice_id;
        $db->setQuery($query);
        $item_name = $item->name . " inv. " . $db->loadResult();
      } else {
        $query = ' SELECT invoice_num FROM #__invoices_invoices '.
        ' WHERE id = ' . $item->invoice_id;
        $db->setQuery($query);
        $item_name = $item->name . " inv. num " . $db->loadResult();
      }

      $existing_id = 0;
      if ($this->params->get('update', '1')){
        $db = JFactory::getDBO();
        $query = 	' SELECT id FROM #__finances_items '.
        ' WHERE external_type = "com_invoices,2" AND external_id = ' . $item->id ;
        $db->setQuery( $query );
        $existing_id = $db->loadResult();
      }

      $data = array();
      $data['id'] = $existing_id ;
      $data['amount'] = $amount;
      $data['name'] = $item_name;
      $data['date_item'] = $row->invoice_date;
      $data['tax'] = $item->tax;
      $data['category_id'] = $this->params->get('category_id', '1');
      $data['project_id'] = $this->params->get('account_id', '1');
      $data['external_id'] = $item->id;
      $data['external_type'] = "com_invoices,2";
      $data['vat_id'] = "";
      $data['external_ref'] = $item->sku;
      //$data['user_id'] = "";
      $data['description'] = $item->desc;
      $data['long_desc'] = "";

      FinancesHelper::create_expense_item($data);
    }
  }

  /**
  * If the invoice has various taxes, only the first one is computed
  *
  * @param $ids. A String like "id1,id2,id3". The ids are the ids of the taxes applied to the invoice
  * @return mixed. The tax percent of the first id in the string
  */
  private function getTax($ids)
  {
    $ids = explode(",", $ids);
    $taxId = $ids[0];

    $db = JFactory::getDBO();
    $query = 	' SELECT value FROM #__invoices_taxes '.
    ' WHERE id = ' . $taxId . ' AND type = "percent"';
    $db->setQuery( $query );

    return $db->loadResult();
  }

}
